﻿using System.Collections.Generic;
using System.Configuration.Provider;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using NHibernate;
using System.Web.Security;
using System.Collections.Specialized;
using System;
using System.Transactions;
using NHProviderTestHelpers;

namespace NHibernateProvider.Tests.Integration
{
    /// <summary>
    ///This is a test class for NHibernateMembershipProviderTest and is intended
    ///to contain the majority NHibernateMembershipProviderTest Integration Tests.
    /// 
    /// This test requires a properly configured database that is referenced in the 
    /// app.config file included with this package (change it to appropriate values).
    /// 
    /// To fully support the unit tests, your web config will need to include encrypt and decrypt
    /// keys, since some of the functions are not supported with auto-generated keys.  
    ///</summary>
    [TestClass()]
    public class NHibernateMembershipProviderTestsDefault
    {
        private NHibernateMembershipProvider _nhp;
        private static NameValueCollection _testConfig;
        private static List<UserParameters> _testUsers;

        #region Setup and Teardown
        [ClassInitialize()]
        public static void ClassSetup(TestContext testContext)
        {
            //Set up a test configuration to use.
            _testConfig = new NameValueCollection();
            _testConfig.Add("applicationName","NHPTestApp");

            //We will create three test users to work with while
            //exercising our tests.  These will be used for various
            //read and find operations.
            var NHPSetup = new NHibernateMembershipProvider();
            NHPSetup.Initialize("NHibernateMembershipProvider", _testConfig);
            _testUsers = TestUtils.GetTestUsers(5, "Default");
            foreach (var u in _testUsers)
            {
                MembershipCreateStatus status;
                NHPSetup.CreateUser(u.username, u.password, u.email, u.passwordQuestion,
                    u.passwordAnswer, u.isApproved, u.providerUserKey, out status);
            }
        }

        [ClassCleanup()]
        public static void ClassCleanup()
        {
            //We will remove our sample users
            var NHPCleanup = new NHibernateMembershipProvider();
            NHPCleanup.Initialize("NHibernateMembershipProvider", _testConfig);
            foreach (var user in _testUsers)
            {
                NHPCleanup.DeleteUser(user.username, true);
            }
        }

        [TestInitialize()]
        public void TestSetup()
        {
            _nhp = new NHibernateMembershipProvider();
            _nhp.Initialize("NHibernateMembershipProvider", _testConfig);
        }

        [TestCleanup()]
        public void TestCleanup()
        {
            _nhp = null;
        }
        #endregion

        #region Test Initialize Method
        [TestMethod]
        public void Initialize_ValidParms_DoesNotThrowException()
        {
            //Arrange
            var nhpTemp = new NHibernateMembershipProvider();
            //Act
            nhpTemp.Initialize("NHibernateMembershipProvider", _testConfig);
            //Assert
            Assert.IsNotNull(nhpTemp.UserRepo);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Initialize_EmptyConfig_ThrowsException()
        {
            //Arrange
            var nhpTemp = new NHibernateMembershipProvider();
            //Act
            nhpTemp.Initialize("NHibernateMembershipProvider", null);
        }

        [TestMethod]
        public void Initialize_NullName_SetsDefault()
        {
            //Arrange
            var nhpTemp = new NHibernateMembershipProvider();
            //Act
            nhpTemp.Initialize(null, _testConfig);
            //Assert
            Assert.IsNotNull(nhpTemp.UserRepo);
        }
        #endregion

        #region Test CreateUser Method
        [TestMethod()]
        public void CreateUserTest_ValidTestUser_ReturnsSuccess()
        {
            UserParameters u = TestUtils.GetValidUser();
            MembershipCreateStatus status = new MembershipCreateStatus();
            MembershipCreateStatus statusExpected = MembershipCreateStatus.Success;
            MembershipUser newuser = _nhp.CreateUser(u.username, u.password, u.email, u.passwordQuestion,
                u.passwordAnswer, u.isApproved, u.providerUserKey, out status);
            //Cleanup
            if (statusExpected == status) _nhp.DeleteUser(u.username,true);
            Assert.AreEqual(statusExpected, status);
        }

        [TestMethod()]
        public void CreateUserTest_InvalidPassword_ReturnsSuccess()
        {
            UserParameters u = TestUtils.GetValidUser();
            MembershipCreateStatus status = new MembershipCreateStatus();
            MembershipCreateStatus statusExpected = MembershipCreateStatus.Success;
            MembershipUser newuser = _nhp.CreateUser(u.username, u.password, u.email, u.passwordQuestion,
                u.passwordAnswer, u.isApproved, u.providerUserKey, out status);
            //Cleanup
            if (statusExpected == status) _nhp.DeleteUser(u.username, true);
            Assert.AreEqual(statusExpected, status);
        }

        [TestMethod()]
        public void CreateUserTest_DuplicateUserName_ReturnsDuplicateUserName()
        {
            UserParameters u = TestUtils.GetValidUser();
            u.username = _testUsers[0].username;
            MembershipCreateStatus status = new MembershipCreateStatus();
            MembershipCreateStatus statusExpected = MembershipCreateStatus.DuplicateUserName;
            MembershipUser newuser = _nhp.CreateUser(u.username, u.password, u.email, u.passwordQuestion,
                u.passwordAnswer, u.isApproved, u.providerUserKey, out status);
            Assert.AreEqual(statusExpected, status);
        }

        [TestMethod()]
        public void CreateUserTest_DuplicateEMail_ReturnsDuplicateEmail()
        {
            UserParameters u = TestUtils.GetValidUser();
            u.email = _testUsers[0].email;
            MembershipCreateStatus status = new MembershipCreateStatus();
            MembershipCreateStatus statusExpected = MembershipCreateStatus.DuplicateEmail;
            MembershipUser newuser = _nhp.CreateUser(u.username, u.password, u.email, u.passwordQuestion,
                u.passwordAnswer, u.isApproved, u.providerUserKey, out status);
            Assert.AreEqual(statusExpected, status);
        }

        [TestMethod()]
        public void CreateUserTest_InvalidPassword_ReturnsInvalidPassword()
        {
            UserParameters u = TestUtils.GetValidUser();
            u.password = "";
            MembershipCreateStatus status = new MembershipCreateStatus();
            MembershipCreateStatus statusExpected = MembershipCreateStatus.InvalidPassword;
            MembershipUser newuser = _nhp.CreateUser(u.username, u.password, u.email, u.passwordQuestion,
                u.passwordAnswer, u.isApproved, u.providerUserKey, out status);
            Assert.AreEqual(statusExpected, status);
        }
        #endregion

        #region Test ChangePassword Method
        [TestMethod()]
        public void ChangePasswordTest_UnapprovedUser_ReturnsFalse()
        {
            UserParameters u = _testUsers[0];
            //Change user to unapproved
            var user = _nhp.GetUser(u.username, true);
            user.IsApproved = false;          
            _nhp.UpdateUser(user);

            string newpass = "!Password??999";
            bool result = true;
            result = _nhp.ChangePassword(u.username, u.password, newpass);
            //Cleanup
            if (result) _nhp.ChangePassword(u.username, newpass, u.password);
            user.IsApproved = true;
            _nhp.UpdateUser(user);

            Assert.IsFalse(result);
        }

        [TestMethod()]
        public void ChangePasswordTest_ValidUser_ReturnsTrue()
        {
            UserParameters u = _testUsers[0];
            string newpass = "!Password??999";
            bool result = false;
            result = _nhp.ChangePassword(u.username, u.password, newpass);
            //Cleanup
            if (result) _nhp.ChangePassword(u.username, newpass, u.password);
            Assert.IsTrue(result);
        }

        [TestMethod()]
        public void ChangePasswordTest_ValidUser_PasswordChanged()
        {
            UserParameters uparm = _testUsers[2];
            string newpass = "!Password??999";
            _nhp.ChangePassword(uparm.username, uparm.password, newpass);
            string curpass = _nhp.GetPassword(uparm.username, uparm.passwordAnswer);
            //Cleanup
            _nhp.ChangePassword(uparm.username, newpass, uparm.password);
            Assert.AreEqual(newpass,curpass);
        }

        [TestMethod()]
        [ExpectedException(typeof(MembershipPasswordException))]
        public void ChangePasswordTest_BadNewPass_ThrowsException()
        {
            UserParameters u = _testUsers[0];
            string newpass = "";
            _nhp.ChangePassword(u.username, u.password, newpass);
        }

        [TestMethod()]
        public void ChangePasswordTest_SamePasswords_ReturnsTrue()
        {
            UserParameters u = _testUsers[0];
            bool result = false;
            result = _nhp.ChangePassword(u.username, u.password, u.password);
            Assert.IsTrue(result);
        }

        [TestMethod()]
        public void ChangePasswordTest_BadOldPass_ReturnsFalse()
        {
            UserParameters u = _testUsers[0];
            string newpass = "!Password??999";
            string badpass = "!!!!BadPass999";
            bool result = true;
            result = _nhp.ChangePassword(u.username, badpass, newpass);
            Assert.IsFalse(result);
        }
        #endregion

        #region Test ChangePasswordQuestionAndAnswer Method
        [TestMethod()]
        public void ChangePasswordQuestionAndAnswer_ValidUser_ReturnsTrue()
        {
            UserParameters u = _testUsers[0];
            string newquestion = "question";
            string newanswer = "answer";
            bool result = false;
            result = _nhp.ChangePasswordQuestionAndAnswer(u.username, u.password, newquestion, newanswer);
            //Cleanup
            if (result) _nhp.ChangePasswordQuestionAndAnswer(u.username, u.password, u.passwordQuestion, u.passwordAnswer);
            Assert.IsTrue(result);
        }

        [TestMethod()]
        public void ChangePasswordQuestionAndAnswer_ValidUser_QandAChanged()
        {
            UserParameters uparm = _testUsers[3];
            User u = _nhp.UserRepo.GetUserByName(uparm.username, _nhp.ApplicationName);
            var oldQA = new PasswordQandA() 
                {Answer = u.PasswordAnswer, Question = u.PasswordQuestion};
            var newQA = new PasswordQandA() 
                {Answer = "Lettuce And Carrots", Question = "What Do Rabbits Eat?" };
            
            _nhp.ChangePasswordQuestionAndAnswer(uparm.username, uparm.password, newQA.Question, newQA.Answer);
            User uCur = _nhp.UserRepo.GetUserByName(uparm.username, _nhp.ApplicationName);
            oldQA.Question = uCur.PasswordQuestion;
            oldQA.Answer = uCur.PasswordAnswer;

            //Cleanup
            _nhp.ChangePasswordQuestionAndAnswer(uparm.username, uparm.password, uparm.passwordQuestion, uparm.passwordAnswer);
            Assert.AreEqual(oldQA, newQA);            
        }


        [TestMethod()]
        public void ChangePasswordQuestionAndAnswer_BadPassword_ReturnsFalse()
        {
            UserParameters u = _testUsers[0];
            string badpass = "BadPassword";
            bool result = false;
            result = _nhp.ChangePasswordQuestionAndAnswer(u.username, badpass, u.passwordQuestion, u.passwordAnswer);
            //Cleanup
            Assert.IsFalse(result);
        }        

        #endregion

        #region Test FindUsersByEMail Method
        [TestMethod()]
        public void FindUsersByEmail_ValidUser_ReturnsOneRecord()
        {
            UserParameters u = _testUsers[0];
            int total = 0;
            MembershipUserCollection users = _nhp.FindUsersByEmail(u.email, 0, 5, out total);
            Assert.AreEqual(total,1);
        }

        [TestMethod()]
        public void FindUsersByEmail_InvalidUser_ReturnsNoRecords()
        {
            string email = "InvalidEmailAddress";
            int total = -1;
            MembershipUserCollection users = _nhp.FindUsersByEmail(email, 0, 5, out total);
            Assert.AreEqual(total, 0);
        }
        #endregion

        #region Test FindUsersByName Method
        [TestMethod()]
        public void FindUsersByName_ValidUser_ReturnsOneRecord()
        {
            UserParameters u = _testUsers[0];
            int total = 0;
            MembershipUserCollection users = _nhp.FindUsersByName(u.username, 0, 5, out total);
            Assert.AreEqual(total, 1);
        }

        [TestMethod()]
        public void FindUsersByName_InvalidUser_ReturnsNoRecords()
        {
            string badname = "InvalidUserName";
            int total = -1;
            MembershipUserCollection users = _nhp.FindUsersByName(badname, 0, 5, out total);
            Assert.AreEqual(total, 0);
        }
        #endregion    

        #region Test GetAllUsers Method
        [TestMethod()]
        public void GetAllUsers_FourPerPage_ReturnsFourRecords()
        {
            UserParameters u = _testUsers[0];
            int total = 0; 
            //We should at least get four of our five test users
            MembershipUserCollection users = _nhp.GetAllUsers(0, 4, out total);
            Assert.AreEqual(total, 4);
        }
        
        #endregion

        #region Test GetNumberOfUsersOnline Method
        [TestMethod()]
        public void GetNumberOfUsersOnline_FourPerPage_ReturnsFourRecords()
        {
            UserParameters u = _testUsers[0];
            int total = -1;
            //We should at least get four of our five test users
            total = _nhp.GetNumberOfUsersOnline();
            Assert.IsFalse(total < 0);
        }
        #endregion 

        #region Test GetPassword Method
        [TestMethod()]
        public void GetPassword_ValidAnswer_ReturnsGoodPassword()
        {
            UserParameters u = _testUsers[0];
            string password;
            password = _nhp.GetPassword(u.username, u.passwordAnswer);
            Assert.AreEqual(password, u.password);
        }

        [TestMethod()]
        public void GetPassword_AnswerNotRequired_ReturnsGoodPassword()
        {
            UserParameters u = _testUsers[0];
            string answer = "KittyCatsLikeTuna";
            string password;
            password = _nhp.GetPassword(u.username, answer);
            Assert.AreEqual(password, u.password);
        }

        [TestMethod()]
        [ExpectedException(typeof(MembershipPasswordException))]
        public void GetPassword_BadAnswer_ThrowsException()
        {
            UserParameters u = _testUsers[0];
            var nhTemp = new NHibernateMembershipProvider();
            nhTemp.Initialize("NHibernateMembershipProvider",TestUtils.GetComplexConfig());
            string answer = "KittyCatsLikeTuna";
            nhTemp.GetPassword(u.username, answer);
        }

        [TestMethod()]
        [ExpectedException(typeof(ProviderException))]
        public void GetPassword_NoRetrieval_ThrowsException()
        {
            UserParameters u = _testUsers[0];
            var nhTemp = new NHibernateMembershipProvider();
            nhTemp.Initialize("NHibernateMembershipProvider", TestUtils.GetNoPasswordConfig());
            string answer = "KittyCatsLikeTuna";
            nhTemp.GetPassword(u.username, answer);
        } 
        #endregion 

        #region Test ValidateUser Method
        [TestMethod()]
        public void ValidateUser_GoodPassword_ReturnsTrue()
        {
            UserParameters u = _testUsers[0];
            bool result;
            result = _nhp.ValidateUser(u.username, u.password);
            Assert.IsTrue(result);
        }

        [TestMethod()]
        public void ValidateUser_BadPassword_ReturnsFalse()
        {
            UserParameters u = _testUsers[0];
            bool result;
            result = _nhp.ValidateUser(u.username, String.Empty);
            Assert.IsFalse(result);
        }


        [TestMethod()]
        public void ValidateUser_BadUser_ReturnsFalse()
        {
            bool result;
            result = _nhp.ValidateUser("TheKingOfFrance", String.Empty);
            Assert.IsFalse(result);
        }

        #endregion

        #region Test GetUser Method
        [TestMethod()]
        public void GetUser_ValidUser_ReturnsObject()
        {
            UserParameters u = _testUsers[0];
            int total;
            MembershipUserCollection users = _nhp.FindUsersByName(u.username, 0, 5, out total);
            MembershipUser mu;
            mu = _nhp.GetUser(users[u.username].ProviderUserKey, true);
            Assert.AreEqual(mu.UserName,u.username);
        }

        #endregion

        #region Test ResetPassword Method
        [TestMethod()]
        public void ResetPassword_GoodUser_ReturnsNewPassword()
        {
            UserParameters u = _testUsers[1];
            string newPass ="";
            newPass = _nhp.ResetPassword(u.username, u.passwordAnswer);
            _nhp.ChangePassword(u.username, newPass, u.password);
            Assert.AreNotEqual(newPass, "");
        }

        [TestMethod()]
        [ExpectedException(typeof(MembershipPasswordException))]
        public void ResetPassword_BadUser_ThrowsException()
        {
            string baduser = "InvalidUser";
            string newPass = "";
            newPass = _nhp.ResetPassword(baduser, baduser);
        }

        [TestMethod()]
        [ExpectedException(typeof(MembershipPasswordException))]
        public void ResetPassword_BadAnswer_ThrowsException()
        {
            UserParameters u = _testUsers[0]; 
            string badanswer = "BadPassword";
            string newPass = "";
            var nhTemp = new NHibernateMembershipProvider();
            nhTemp.Initialize("NHibernateMembershipProvider", TestUtils.GetComplexConfig());
            newPass = nhTemp.ResetPassword(u.username, badanswer);
        }

        [TestMethod()]
        [ExpectedException(typeof(ProviderException))]
        public void ResetPassword_NullAnswer_ThrowsException()
        {
            UserParameters u = _testUsers[0];
            string newPass = "";
            var nhTemp = new NHibernateMembershipProvider();
            nhTemp.Initialize("NHibernateMembershipProvider", TestUtils.GetComplexConfig());
            newPass = nhTemp.ResetPassword(u.username, null);
        }

        [TestMethod()]
        public void ResetPassword_AnswerNotRequired_ReturnsNewPassword()
        {
            UserParameters u = _testUsers[0];
            string badanswer = "BadPassword";
            string newPass = "";
            newPass = _nhp.ResetPassword(u.username, badanswer);
            _nhp.ChangePassword(u.username, newPass, u.password);
            Assert.AreNotEqual(newPass, "");
        }
        #endregion 

        #region Test UnlockUser Method
        [TestMethod()]
        public void UnlockUser_ValidUser_ReturnsTrue()
        {
            UserParameters u = _testUsers[4];
            bool result = false;
            result = _nhp.UnlockUser(u.username);
            Assert.IsTrue(result);
        }

        [TestMethod()]
        public void UnlockUser_InvalidUser_ReturnsTrue()
        {
            bool result = false;
            result = _nhp.UnlockUser("TheKingOfFrance");
            Assert.IsTrue(result);
        }

        #endregion 

        #region Test Properties

        [TestMethod()]
        public void Properties_ExerciseAllGetsAndSets()
        {
            var nhTemp = new NHibernateMembershipProvider();
            nhTemp.Initialize("NHibernateMembershipProvider", TestUtils.GetComplexConfig());
            nhTemp.ApplicationName = "TempApp";
            Assert.AreEqual(nhTemp.ApplicationName, "TempApp");
            Assert.AreEqual(nhTemp.MaxInvalidPasswordAttempts, 3);
            Assert.AreEqual(nhTemp.MinRequiredNonAlphanumericCharacters, 1);
            Assert.AreEqual(nhTemp.MinRequiredPasswordLength, 7);
            Assert.AreEqual(nhTemp.PasswordAttemptWindow, 10);
            Assert.AreEqual(nhTemp.PasswordStrengthRegularExpression, "^.*(?=.{6,})(?=.*[a-z])(?=.*[A-Z]).*$");
        }

        #endregion
    }
}
